
// Klasa reprezentująca wykres.
function Plot ()
{
	// Szerokość pola graficznego.
	this.width = 1000;
	// Wysokość pola graficznego.
	this.height = 400;
	// Lewy margines.
	this.gaMarginLeft = 100;
	// Prawy margines.
	this.gaMarginRight = 100;
	// Górny margines.
	this.gaMarginTop = 20;
	// Dolny margines.
	this.gaMarginBottom = 30;
	// Szerokość wykresu.
	this.gaWidth = this.width - this.gaMarginLeft - this.gaMarginRight;
	// Wysokość wykresu.
	this.gaHeight = this.height - this.gaMarginTop - this.gaMarginBottom;
	// Tablica na wektory współrzędnych X.
	this.x = [];
	// Tablica na wektory współrzędnych Y.
	this.y = [];
	// Liczba segmentów podziału osi X.
	this.xTics = 10;
	// Liczba segmentów podziału osi Y.
	this.yTics = 4;
	// Tablica etykiet do wykresów.
	this.label = [];
	// Minimalna wielkość współrzędnej X.
	this.minX = 0;
	// Maksymalna wielkość współrzędnej X.
	this.maxX = 0;
	// Odległość między skrajnymi wartościami na osi X.
	this.deltaX = 0;
	// Maksymalna bezwzględna wartość współrzędnej X.
	this.maxAbsX = 0;
	// Minimalna wartość współrzędnej Y.
	this.minY = 0;
	// Maksymalna wartość współrzędnej Y.
	this.maxY = 0;
	// Odległość między skrajnymi wartościami na osi Y.
	this.deltaY = 0;
	// Maksymalna bezwzględna wartość współrzędnej Y.
	this.maxAbsY = 0;
	// Użyj notacji wykładniczej na osi X.
	this.useExponentialX = false;
	// Użyj notacji wykładniczej na osi Y.
	this.useExponentialY = false;
	// Grubość lini wykresu.
	this.lineWidth = 1;
}

// Zmiana wymiarów płótna, na którym rysowany jest wykres.
Plot.prototype.resize = function (width, height)
{
	this.width = width;
	this.height = height;
	this.gaWidth = this.width - this.gaMarginLeft - this.gaMarginRight;
	this.gaHeight = this.height - this.gaMarginTop - this.gaMarginBottom;
}

Plot.prototype.findMinMaxX = function ()
{
	// Wyznaczenie wartości minimalnej i maksymalnej dla osi X.
	if (this.x.length === 1) {
		this.minX = this.x[0][0];
		this.maxX = this.minX;
	}
	
	var i = this.x.length - 1;
	for (var j = 0; j < this.x[i].length; j++) {
		var t = this.x[i][j];
		if (t < this.minX) this.minX = t;
		if (this.maxX < t) this.maxX = t;
	}
	
	if (this.minX === this.maxX) {
		this.maxX += 1;
	}
	
	this.deltaX = this.maxX - this.minX;
	
	this.maxAbsX = Math.abs(this.maxX);
	if (this.maxAbsX < Math.abs(this.minX)) {
		this.maxAbsX = Math.abs(this.minX);
	}
}

Plot.prototype.findMinMaxY = function ()
{
	// Wyznaczenie wartości minimalnej i maksymalnej dla osi Y.
	if (this.y.length === 1) {
		this.minY = this.y[0][0];
		this.maxY = this.minY;
	}
	
	var i = this.y.length - 1;
	for (var j = 0; j < this.y[i].length; j++) {
		var t = this.y[i][j];
		if (t < this.minY) this.minY = t;
		if (this.maxY < t) this.maxY = t;
	}

	if (this.minY === this.maxY) {
		this.maxY += 1;
	}
	
	this.deltaY = this.maxY - this.minY;

	this.maxAbsY = Math.abs(this.maxY);
	if (this.maxAbsY < Math.abs(this.minY)) {
		this.maxAbsY = Math.abs(this.minY);
	}
}

// Dodanie danych do wykresu. Można wywoływać wielokrotnie dodając 
// kolejne wykresy.
//
// Argumenty:
// y - tablica współrzędnych Y,
// x - tablica współrzędnych X (ten argument może być pusty, wtedy 
//     implementacja przyjmie domyślne wartości 0, 1, 2, ..., n),
Plot.prototype.addData = function (y, x)
{
	// Przypisanie wartości.
	this.y.push(y);
	
	// Przypisanie argumentów, jażeli zostały podane.
	if (arguments.length === 2) {
		this.x.push(x);
	// Przypisanie argumentów domyślnych: 0, 1, 2, ..., n.
	} else {
		this.x[this.x.length] = [];
		for (var i = 0; i < y.length; i++) {
			this.x[this.x.length-1].push(i);
		}
	}
	
	this.findMinMaxX();
	this.findMinMaxY();
}

// Etykieta dla wykresu.
Plot.prototype.addLabel = function (label)
{
	this.label.push(label);
}

// Etykieta na osi X.
Plot.prototype.putLabelX = function(x, y, text, toExp)
{
	var str = "<text fill='gray' text-anchor='middle' " +
		"x='" + x + "' y='" + (y + 20) + "'>" + (toExp ? text.toExponential(2) : text.toFixed(2)) + "</text>" +
		"<line x1='" + x + "' y1='" + y + "' x2='" + x + "' y2='" +
		(y + 5) + "' style='fill:none;stroke:black;stroke-width:1;opacity:1.0'/>";
	return str;
}

// Etykieta na osi Y.
Plot.prototype.putLabelY = function(x, y, text, toExp)
{
	var str = "<text fill='gray' text-anchor='end' " +
		"x='" + (x - 10) + "' y='" + y + "'>" + (toExp ? text.toExponential(2) : text.toFixed(2)) + "</text>" +
		"<line x1='" + x + "' y1='" + y + "' x2='" + (x - 5) + "' y2='" +
		y + "' style='fill:none;stroke:black;stroke-width:1;opacity:1.0'/>";
	return str;
}

// Kod HTML służący do wygenerowania ramki wykresu.
Plot.prototype.svgFramePath = function ()
{
	// Ramka.
	var frame = "<path d='M " + this.gaMarginLeft + " " + this.gaMarginTop +
		" L " + this.gaMarginLeft + " " + (this.height - this.gaMarginBottom) +
		" L " + (this.width - this.gaMarginRight) + "  " + (this.height - this.gaMarginBottom) +
		" L " + (this.width - this.gaMarginRight) + "  " + this.gaMarginTop +
		" L " + this.gaMarginLeft + " " + this.gaMarginTop + "' " +
		"style='fill:none;stroke:black;stroke-width:1;opacity:1.0'/>";
	
	var nx = this.xTics;
	var dx = (this.width - this.gaMarginLeft - this.gaMarginRight) / nx;
	for (var i = 0; i < nx+1; i++) {
		frame += this.putLabelX(this.gaMarginLeft + i*dx, this.height - this.gaMarginBottom, this.minX + i*this.deltaX/nx, this.useExponentialX);
	}
	
	var ny = this.yTics;
	var dy = (this.height - this.gaMarginTop - this.gaMarginBottom) / ny;
	for (var i = 0; i < ny+1; i++) {
		frame += this.putLabelY(this.gaMarginLeft, this.gaMarginTop + i*dy, this.maxY - i*this.deltaY/ny, this.useExponentialY);
	}
	
	return frame;
}

// Kod HTML służący do wygenerowania wszystkich wykresów (każdy wykres 
// jest zdefiniowany jako niezależna ścieżka).
Plot.prototype.svgGraphPath = function ()
{
	function makePlotStyle (color, opacity, lineWidth) {
		return "style='fill:none;stroke:" + color + ";stroke-width:" + lineWidth + ";opacity:" + opacity + "'";
	}
	
	var plotColor = [
		"red",
		"blue",
		"lime",
		"purple",
		"gold",
		"magenta",
		"teal",
		"orangered"
	];
	
	// Wygenerowanie wykresów z odpowiednim przeskalowaniem.
	var plots = [];
	
	for (var i = 0; i < this.x.length; i++) {
		plots[i] = "<path d='M ";
		var x, y;
		for (var j = 0; j < this.x[i].length; j++) {
			x = this.gaMarginLeft + Math.round((this.x[i][j] - this.minX) / this.deltaX * this.gaWidth);
			plots[i] += x;
			plots[i] += " ";
			y = this.gaMarginTop + Math.round((1 - (this.y[i][j] - this.minY) / this.deltaY) * this.gaHeight);
			plots[i] += y;
			if (j !== this.x[i].length-1) {
				plots[i] += " L ";
			}
		}
		var colorNum = i % 8;
		plots[i] += "' " + makePlotStyle(plotColor[colorNum], 0.9, this.lineWidth) + "/>";
		if (this.label[i] !== "") {
			plots[i] += "<text fill='" + plotColor[colorNum] +"' font-style='italic' text-anchor='start' " +
			"x='" + (x + 10) + "' y='" + y + "'>" + this.label[i] + "</text>";
		}
	}
	
	return plots;
}
