
function PlotPolar () {
	Plot.call(this);
	this.rTics = 4;
	this.aTics = 12;
	// Użyj notacji wykładniczej dla promienia R.
	this.useExponentialR = false;
}

PlotPolar.prototype = Object.create(Plot.prototype);

PlotPolar.prototype.constructor = PlotPolar;

// Dodanie danych do wykresu. Można wywoływać wielokrotnie dodając 
// kolejne wykresy.
//
// Argumenty:
// r - tablica promieni wodzących,
// a - tablica kątów (ten argument może być pusty, wtedy implementacja 
//     przyjmie domyślne wartości 2*pi*k / n),
PlotPolar.prototype.addData = function (r, a)
{
	// Wyznaczenie domyślnej tablicy kątów w przypadku jej braku.
	if (arguments.length === 1) {
		var a = [];
		for (var k = 0; k < r.length; k++) {
			a[k] = 2*Math.PI*k / r.length;
		}
	}
	
	y = [];
	x = [];
	
	// Przeniesienie danych do układu kartezjańskiego.
	for (var k = 0; k < r.length; k++) {
		x[k] = r[k] * Math.cos(a[k]);
		y[k] = r[k] * Math.sin(a[k]);
	}
	
	this.y.push(y);
	this.x.push(x);
	
	this.findMinMaxX();
	this.findMinMaxY();
}

// Etykieta promienia.
Plot.prototype.putLabelR = function(x, y, text, toExp)
{
	var str = "<text fill='black' text-anchor='right' " +
		"x='" + (x + 5) + "' y='" + (y - 5) + "'>" + (toExp ? text.toExponential(2) : text.toFixed(2)) + "</text>";
	return str;
}

// Etykieta kąta.
Plot.prototype.putLabelAlpha = function(x, y, text, toExp)
{
	var str = "<text fill='black' text-anchor='end' " +
		"x='" + x + "' y='" + y + "'>" + (toExp ? text.toExponential(2) : text.toFixed(2)) + "</text>";
	return str;
}

PlotPolar.prototype.svgFramePath = function () {
	// Promień.
	var R = 0.5 * this.gaHeight;
	
	if (0.5 * this.gaWidth < R) {
		R = 0.5 * this.gaWidth;
	}
	
	// Środek.
	var Ox = this.gaMarginLeft + 0.5*this.gaWidth;
	var Oy = this.gaMarginTop + 0.5*this.gaHeight;
	
	// Maksymalna wartość na którejkolwiek ze współrzędnych.
	var maxAbsXY = this.maxAbsX < this.maxAbsY ? this.maxAbsY : this.maxAbsX;

	// Ramka.
	var frame = "";
	
	// Okręgi współśrodkowe.
	for (var k = 0; k < this.rTics; k++) {
		var r = ((k+1) / this.rTics) * (R - 20)
		frame += "<circle cx='" + Ox + "' cy='" + Oy + "' r='" + r + "' " +
			"style='fill:none;stroke:black;stroke-width:1;opacity:1.0'/>";
		frame += this.putLabelR(Ox, Oy - r, r*maxAbsXY / R, this.useExponentialR);
	}
	
	// Promienie.
	for (var k = 0; k < this.aTics; k++) {
		var sinA = Math.sin(k * (2*Math.PI / this.aTics));
		var cosA = Math.cos(k * (2*Math.PI / this.aTics));
		frame += "<line x1='" + Ox + "' y1='" + Oy + 
			"' x2='" + (Ox + R*cosA) + "' y2='" + (Oy + R*sinA) + "' " +
			"style='fill:none;stroke:black;stroke-width:1;opacity:1.0'/>";
	}
	
	return frame;
}

// Kod HTML służący do wygenerowania wszystkich wykresów (każdy wykres 
// jest zdefiniowany jako niezależna ścieżka).
PlotPolar.prototype.svgGraphPath = function ()
{
	function makePlotStyle (color, opacity, lineWidth) {
		return "style='fill:none;stroke:" + color + ";stroke-width:" + lineWidth + ";opacity:" + opacity + "'";
	}
	
	var plotColor = [
		"red",
		"blue",
		"lime",
		"purple",
		"gold",
		"magenta",
		"teal",
		"orangered"
	];
	
	// Promień.
	var R = 0.5 * this.gaHeight;
	
	if (0.5 * this.gaWidth < R) {
		R = 0.5 * this.gaWidth;
	}
	
	R *= 0.9;
	
	// Środek.
	var Ox = Math.round(this.gaMarginLeft + 0.5*this.gaWidth);
	var Oy = Math.round(this.gaMarginTop + 0.5*this.gaHeight);
	
	// Maksymalna wartość na którejkolwiek ze współrzędnych.
	var maxAbsXY = this.maxAbsX < this.maxAbsY ? this.maxAbsY : this.maxAbsX;
	
	// Wygenerowanie wykresów z odpowiednim przeskalowaniem.
	var plots = [];
	
	for (var i = 0; i < this.x.length; i++) {
		plots[i] = "<path d='M ";
		var x, y;
		for (var j = 0; j < this.x[i].length; j++) {
			x = Ox + R*(this.x[i][j] / maxAbsXY);
			plots[i] += x;
			plots[i] += " ";
			y = Oy - R*(this.y[i][j] / maxAbsXY);
			plots[i] += y;
			if (j !== this.x[i].length-1) {
				plots[i] += " L ";
			}
		}
		var colorNum = i % 8;
		plots[i] += " Z' " + makePlotStyle(plotColor[colorNum], 0.9, this.lineWidth) + "/>";
		if (this.label[i] !== "") {
			plots[i] += "<text fill='" + plotColor[colorNum] +"' font-size=23px font-weight='bold' font-style='italic' text-anchor='start' " +
			"x='" + (x + 30) + "' y='" + (y - 5) + "'>" + this.label[i] + "</text>";
		}
	}
	
	return plots;
}
